@@ -12,9 +12,9 @@ use syn::{
 use crate::args::{self, ComplexityType, RenameRuleExt, RenameTarget};
 use crate::output_type::OutputType;
 use crate::utils::{
-    extract_input_args, gen_deprecation, generate_default, generate_guards, generate_validator,
-    get_cfg_attrs, get_crate_name, get_param_getter_ident, get_rustdoc, get_type_path_and_name,
-    parse_complexity_expr, parse_graphql_attrs, remove_graphql_attrs, visible_fn, GeneratorResult,
+    extract_input_args, gen_deprecation, generate_default, generate_guards, get_cfg_attrs,
+    get_crate_name, get_rustdoc, get_type_path_and_name, parse_complexity_expr,
+    parse_graphql_attrs, remove_graphql_attrs, visible_fn, GeneratorResult,
 };
 
 pub fn generate(
@@ -205,22 +205,13 @@ pub fn generate(
                     })
                     .unwrap_or_else(|| quote! {::std::option::Option::None});
 
-                let validator = match &validator {
-                    Some(meta) => {
-                        let stream = generate_validator(&crate_name, meta)?;
-                        quote!(::std::option::Option::Some(#stream))
-                    }
-                    None => quote!(::std::option::Option::None),
-                };
-
                 let visible = visible_fn(visible);
                 schema_args.push(quote! {
                     args.insert(#name, #crate_name::registry::MetaInputValue {
                         name: #name,
                         description: #desc,
-                        ty: <#ty as #crate_name::Type>::create_type_info(registry),
+                        ty: <#ty as #crate_name::InputType>::create_type_info(registry),
                         default_value: #schema_default,
-                        validator: #validator,
                         visible: #visible,
                         is_secret: #secret,
                     });
@@ -235,15 +226,19 @@ pub fn generate(
                     }
                     None => quote! { ::std::option::Option::None },
                 };
-                // We're generating a new identifier,
-                // so remove the 'r#` prefix if present
-                let param_getter_name = get_param_getter_ident(&ident.ident.unraw().to_string());
+
+                let validators = validator.clone().unwrap_or_default().create_validators(
+                    &crate_name,
+                    quote!(&#ident),
+                    quote!(ty),
+                    Some(quote!(.map_err(|err| err.into_server_error(__pos)))),
+                )?;
+
                 get_params.push(quote! {
-                        #[allow(non_snake_case)]
-                        let #param_getter_name = || -> #crate_name::ServerResult<#ty> { ctx.param_value(#name, #default) };
-                        #[allow(non_snake_case)]
-                        let #ident: #ty = #param_getter_name()?;
-                    });
+                    #[allow(non_snake_case)]
+                    let (__pos, #ident) = ctx.param_value::<#ty>(#name, #default)?;
+                    #validators
+                });
             }
 
             let schema_ty = ty.value_type();
@@ -310,7 +305,7 @@ pub fn generate(
                         #(#schema_args)*
                         args
                     },
-                    ty: <#schema_ty as #crate_name::Type>::create_type_info(registry),
+                    ty: <#schema_ty as #crate_name::OutputType>::create_type_info(registry),
                     deprecation: #field_deprecation,
                     cache_control: #cache_control,
                     external: #external,
@@ -341,12 +336,15 @@ pub fn generate(
             let resolve_obj = quote! {
                 {
                     let res = self.#field_ident(ctx, #(#use_params),*).await;
-                    res.map_err(|err| ::std::convert::Into::<#crate_name::Error>::into(err).into_server_error(ctx.item.pos))?
+                    res.map_err(|err| ::std::convert::Into::<#crate_name::Error>::into(err).into_server_error(ctx.item.pos))
                 }
             };
 
+            let guard_map_err = quote! {
+                .map_err(|err| err.into_server_error(ctx.item.pos))
+            };
             let guard = match &method_args.guard {
-                Some(meta_list) => generate_guards(&crate_name, meta_list)?,
+                Some(code) => Some(generate_guards(&crate_name, code, guard_map_err)?),
                 None => None,
             };
 
@@ -360,11 +358,14 @@ pub fn generate(
             resolvers.push(quote! {
                 #(#cfg_attrs)*
                 if ctx.item.node.name.node == #field_name {
-                    #(#get_params)*
-                    #guard
+                    let f = async move {
+                        #(#get_params)*
+                        #guard
+                        #resolve_obj
+                    };
+                    let obj = f.await.map_err(|err| ctx.set_error_path(err))?;
                     let ctx_obj = ctx.with_selection_set(&ctx.item.node.selection_set);
-                    let res = #resolve_obj;
-                    return #crate_name::OutputType::resolve(&res, &ctx_obj, ctx.item).await.map(::std::option::Option::Some);
+                    return #crate_name::OutputType::resolve(&obj, &ctx_obj, ctx.item).await.map(::std::option::Option::Some);
                 }
             });
 
